@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2015 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -25,19 +25,20 @@ through Ssh).
 
 A BackupExecutor is invoked by the BackupManager for backup operations.
 """
-from abc import ABCMeta, abstractmethod
+
 import logging
 import os
 import re
+from abc import ABCMeta, abstractmethod
 
-import psycopg2
-
-from barman.command_wrappers import RsyncPgData, CommandFailedException, \
-    Command, DataTransferFailure
-from barman.utils import mkpath
 from barman import output, xlog
+from barman.command_wrappers import (Command, CommandFailedException,
+                                     DataTransferFailure, RsyncPgData)
 from barman.config import BackupOptions
-
+from barman.infofile import BackupInfo
+from barman.postgres import PostgresConnectionError
+from barman.remote_status import RemoteStatusMixin
+from barman.utils import mkpath, with_metaclass
 
 _logger = logging.getLogger(__name__)
 
@@ -48,13 +49,11 @@ class SshCommandException(Exception):
     """
 
 
-class BackupExecutor(object):
+class BackupExecutor(with_metaclass(ABCMeta, RemoteStatusMixin)):
     """
     Abstract base class for any backup executors.
     """
 
-    __metaclass__ = ABCMeta
-
     def __init__(self, backup_manager):
         """
         Base constructor
@@ -62,6 +61,7 @@ class BackupExecutor(object):
         :param barman.backup.BackupManager backup_manager: the BackupManager
             assigned to the executor
         """
+        super(BackupExecutor, self).__init__()
         self.backup_manager = backup_manager
         self.server = backup_manager.server
         self.config = backup_manager.config
@@ -92,12 +92,15 @@ class BackupExecutor(object):
         Set additional status info - invoked by BackupManager.status()
         """
 
-    def get_remote_status(self):
+    def fetch_remote_status(self):
         """
         Get additional remote status info - invoked by
         BackupManager.get_remote_status()
 
-        :rtype: dict[str, str]
+        This method does not raise any exception in case of errors,
+        but set the missing values to None in the resulting dictionary.
+
+        :rtype: dict[str, None|str]
         """
         return {}
 
@@ -121,7 +124,7 @@ def _parse_ssh_command(ssh_command):
     return ssh_command, ssh_options
 
 
-class SshBackupExecutor(BackupExecutor):
+class SshBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
     """
     Abstract base class for any backup executors based on Ssh
     remote connections. This class is also a factory for
@@ -130,8 +133,6 @@ class SshBackupExecutor(BackupExecutor):
     Raises a SshCommandException if 'ssh_command' is not set.
     """
 
-    __metaclass__ = ABCMeta
-
     def __init__(self, backup_manager):
         """
         Constructor of the abstract class for backups via Ssh
@@ -199,8 +200,11 @@ class SshBackupExecutor(BackupExecutor):
         :param barman.infofile.BackupInfo backup_info: backup information
         """
 
+        previous_backup = self.backup_manager.get_previous_backup(
+            backup_info.backup_id)
+
         # Start the backup, all the subsequent code must be wrapped in a
-        # try except block which finally issues a backup_stop command
+        # try except block which finally issues a stop_backup command
         try:
             self.strategy.start_backup(backup_info)
         except BaseException:
@@ -212,22 +216,31 @@ class SshBackupExecutor(BackupExecutor):
             # This must be inside the try-except, because it could fail
             backup_info.save()
 
-            # If this is the first backup, purge unused WAL files
-            previous_backup = self.backup_manager.get_previous_backup(
-                backup_info.backup_id)
-            if not previous_backup:
-                self.backup_manager.remove_wal_before_backup(backup_info)
-
             output.info("Backup start at xlog location: %s (%s, %08X)",
                         backup_info.begin_xlog,
                         backup_info.begin_wal,
                         backup_info.begin_offset)
 
+            # If this is the first backup, purge eventually unused WAL files
+            if not previous_backup:
+                output.info("This is the first backup for server %s",
+                            self.config.name)
+                removed = self.backup_manager.remove_wal_before_backup(
+                    backup_info)
+                if removed:
+                    output.info("WAL segments preceding the current backup "
+                                "have been found:", log=False)
+                    for wal_name in removed:
+                        output.info("\t%s from server %s "
+                                    "has been removed",
+                                    wal_name, self.config.name)
+
             # Start the copy
             self.current_action = "copying files"
             output.info("Copying files.")
             # perform the backup copy, honouring the retry option if set
-            self.backup_manager.retry_backup_copy(self.backup_copy, backup_info)
+            self.backup_manager.retry_backup_copy(self.backup_copy,
+                                                  backup_info)
 
             output.info("Copy done.")
         except:
@@ -255,15 +268,52 @@ class SshBackupExecutor(BackupExecutor):
         """
 
         # Execute a 'true' command on the remote server
-        cmd = Command(self.ssh_command, self.ssh_options)
-        ret = cmd("true")
+        # TODO: replace with RemoteUnixCommand
+        cmd = Command(self.ssh_command,
+                      self.ssh_options,
+                      path=self.server.path)
+
         hint = "PostgreSQL server"
-        if ret == 0:
-            check_strategy.result(self.config.name, 'ssh', True,
-                                  hint)
-        else:
-            check_strategy.result(self.config.name, 'ssh', False,
-                                  '%s, return code: %s' % (hint, ret))
+
+        # Try executing 'true' through the ssh connection
+        return_code = None
+        try:
+            return_code = cmd("true")
+        except OSError as e:
+            # The execution of ssh command has failed
+            hint = "error executing '%s': %s" % (self.ssh_command, e.strerror)
+
+        # The execution of ssh has succeeded but the return code is not 0
+        if return_code is not None and return_code != 0:
+            hint = '%s, return code: %s' % (hint, return_code)
+
+        # Output the result
+        check_strategy.result(self.config.name, 'ssh', return_code == 0, hint)
+
+        # If SSH works but PostgreSQL is not responding
+        if (return_code == 0 and
+                self.server.get_remote_status()['server_txt_version']
+                is None):
+            # Check for 'backup_label' presence
+            last_backup = self.server.get_backup(
+                self.server.get_last_backup_id(BackupInfo.STATUS_NOT_EMPTY)
+            )
+            # Look for the latest backup in the catalogue
+            if last_backup:
+                # Get PGDATA and build path to 'backup_label'
+                backup_label = os.path.join(last_backup.pgdata,
+                                            'backup_label')
+                # Verify that backup_label exists in the remote PGDATA.
+                # If so, send an alert. Do not show anything if OK.
+                # TODO: replace with RemoteUnixCommand
+                exists = cmd("test -e %s" % backup_label)
+                if exists == 0:
+                    hint = 'Check that the PostgreSQL server is up ' \
+                           "and no 'backup_label' file is in PGDATA."
+                    check_strategy.result(self.config.name,
+                                          'backup_label', False,
+                                          hint)
+
         try:
             # Invoke specific checks for the backup strategy
             self.strategy.check(check_strategy)
@@ -274,17 +324,9 @@ class SshBackupExecutor(BackupExecutor):
     def status(self):
         """
         Set additional status info for SshBackupExecutor using remote
-        commands via Ssh, as well as those defined by the given backup strategy.
+        commands via Ssh, as well as those defined by the given
+        backup strategy.
         """
-        # If the PostgreSQL version is < 9.4 pg_stat_archiver is not available.
-        # Retrieve the last_archived_wal using the executor
-        remote_status = self.get_remote_status()
-        if 'last_archived_wal' in remote_status:
-            output.result('status', self.config.name,
-                          'last_archived_wal',
-                          'Last archived WAL',
-                          remote_status['last_archived_wal'] or
-                          'No WAL segment shipped yet')
         try:
             # Invoke the status() method for the given strategy
             self.strategy.status()
@@ -292,35 +334,44 @@ class SshBackupExecutor(BackupExecutor):
             self._update_action_from_strategy()
             raise
 
-    def get_remote_status(self):
+    def fetch_remote_status(self):
         """
         Get remote information on PostgreSQL using Ssh, such as
         last archived WAL file
 
-        :rtype: dict(str,str|None)
+        This method does not raise any exception in case of errors,
+        but set the missing values to None in the resulting dictionary.
+
+        :rtype: dict[str, None|str]
         """
         remote_status = {}
-        with self.server.pg_connect():
-            # Retrieve the last archived WAL using a Ssh connection on
-            # the remote server and executing an 'ls' command. Only
-            # for pre-9.4 versions of PostgreSQL.
-            if self.server.server_version < 90400:
+        # Retrieve the last archived WAL using a Ssh connection on
+        # the remote server and executing an 'ls' command. Only
+        # for pre-9.4 versions of PostgreSQL.
+        try:
+            if self.server.postgres and \
+                    self.server.postgres.server_version < 90400:
                 remote_status['last_archived_wal'] = None
-                if self.server.get_pg_setting('data_directory') and \
-                        self.server.get_pg_setting('archive_command'):
+                if self.server.postgres.get_setting('data_directory') and \
+                        self.server.postgres.get_setting('archive_command'):
                     # TODO: replace with RemoteUnixCommand
+                    # The Command can raise OSError
+                    # if self.ssh_command does not exist.
                     cmd = Command(self.ssh_command,
-                                  self.ssh_options)
+                                  self.ssh_options,
+                                  path=self.server.path)
                     archive_dir = os.path.join(
-                        self.server.get_pg_setting('data_directory'),
+                        self.server.postgres.get_setting('data_directory'),
                         'pg_xlog', 'archive_status')
-                    out = str(cmd.getoutput('ls', '-tr', archive_dir)[0])
+                    out = str(cmd.getoutput('ls', '-t', archive_dir)[0])
                     for line in out.splitlines():
                         if line.endswith('.done'):
                             name = line[:-5]
                             if xlog.is_any_xlog_file(name):
                                 remote_status['last_archived_wal'] = name
                                 break
+        except (PostgresConnectionError, OSError) as e:
+            _logger.warn("Error retrieving PostgreSQL status: %s", e)
         return remote_status
 
 
@@ -392,7 +443,8 @@ class RsyncBackupExecutor(SshBackupExecutor):
                         tablespace.location[len(backup_info.pgdata):])
                 # Make sure the destination directory exists in order for
                 # smart copy to detect that no file is present there
-                tablespace_dest = backup_info.get_data_directory(tablespace.oid)
+                tablespace_dest = backup_info.get_data_directory(
+                    tablespace.oid)
                 mkpath(tablespace_dest)
                 # Exclude and protect the tablespace from being copied again
                 # during the data directory copy
@@ -401,6 +453,7 @@ class RsyncBackupExecutor(SshBackupExecutor):
                 # tablespace of the previous backup if incremental is active
                 ref_dir = self._reuse_dir(previous_backup, tablespace.oid)
                 tb_rsync = RsyncPgData(
+                    path=self.server.path,
                     ssh=self.ssh_command,
                     ssh_options=self.ssh_options,
                     args=self._reuse_args(ref_dir),
@@ -413,7 +466,7 @@ class RsyncBackupExecutor(SshBackupExecutor):
                         tablespace_dest,
                         safe_horizon,
                         ref_dir)
-                except CommandFailedException, e:
+                except CommandFailedException as e:
                     msg = "data transfer failure on directory '%s'" % \
                           backup_info.get_data_directory(tablespace.oid)
                     raise DataTransferFailure.from_rsync_error(e, msg)
@@ -427,6 +480,7 @@ class RsyncBackupExecutor(SshBackupExecutor):
         # of the previous backup if incremental is active
         ref_dir = self._reuse_dir(previous_backup)
         rsync = RsyncPgData(
+            path=self.server.path,
             ssh=self.ssh_command,
             ssh_options=self.ssh_options,
             args=self._reuse_args(ref_dir),
@@ -437,7 +491,7 @@ class RsyncBackupExecutor(SshBackupExecutor):
             rsync.smart_copy(':%s/' % backup_info.pgdata, backup_dest,
                              safe_horizon,
                              ref_dir)
-        except CommandFailedException, e:
+        except CommandFailedException as e:
             msg = "data transfer failure on directory '%s'" % \
                   backup_info.pgdata
             raise DataTransferFailure.from_rsync_error(e, msg)
@@ -446,7 +500,7 @@ class RsyncBackupExecutor(SshBackupExecutor):
         try:
             rsync(':%s/global/pg_control' % (backup_info.pgdata,),
                   '%s/global/pg_control' % (backup_dest,))
-        except CommandFailedException, e:
+        except CommandFailedException as e:
             msg = "data transfer failure on file '%s/global/pg_control'" % \
                   backup_info.pgdata
             raise DataTransferFailure.from_rsync_error(e, msg)
@@ -458,7 +512,8 @@ class RsyncBackupExecutor(SshBackupExecutor):
             cf = getattr(backup_info, key, None)
             if cf:
                 assert isinstance(cf, str)
-                # Consider only those that reside outside of the original PGDATA
+                # Consider only those that reside outside of the original
+                # PGDATA directory
                 if cf.startswith(backup_info.pgdata):
                     self.current_action = \
                         "skipping %s as contained in %s directory" % (
@@ -470,7 +525,7 @@ class RsyncBackupExecutor(SshBackupExecutor):
                 _logger.info(self.current_action)
                 try:
                     rsync(':%s' % cf, backup_dest)
-                except CommandFailedException, e:
+                except CommandFailedException as e:
                     ret_code = e.args[0]['ret']
                     msg = "data transfer failure on file '%s'" % cf
                     if 'ident_file' == key and ret_code == 23:
@@ -497,7 +552,8 @@ class RsyncBackupExecutor(SshBackupExecutor):
                     "for files that reside outside PGDATA.\n"
                     "Please manually backup the following files:\n"
                     "\t%s\n",
-                    "\n\t".join(filtered_files))
+                    "\n\t".join(filtered_files)
+                )
 
     def _reuse_dir(self, previous_backup_info, oid=None):
         """
@@ -539,13 +595,11 @@ class RsyncBackupExecutor(SshBackupExecutor):
             return []
 
 
-class BackupStrategy(object):
+class BackupStrategy(with_metaclass(ABCMeta, object)):
     """
     Abstract base class for a strategy to be used by a backup executor.
     """
 
-    __metaclass__ = ABCMeta
-
     def __init__(self, executor):
         """
         Constructor
@@ -574,6 +628,7 @@ class BackupStrategy(object):
         :param barman.infofile.BackupInfo backup_info: backup information
         """
 
+    @abstractmethod
     def check(self, check_strategy):
         """
         Perform additional checks - invoked by BackupExecutor.check()
@@ -595,32 +650,30 @@ class BackupStrategy(object):
         :param barman.infofile.BackupInfo backup_info: backup information
         """
         server = self.executor.server
-        with server.pg_connect():
 
-            # Get the PostgreSQL data directory location
-            self.current_action = 'detecting data directory'
-            output.debug(self.current_action)
-            data_directory = server.get_pg_setting('data_directory')
-            backup_info.set_attribute('pgdata', data_directory)
+        # Get the PostgreSQL data directory location
+        self.current_action = 'detecting data directory'
+        output.debug(self.current_action)
+        data_directory = server.postgres.get_setting('data_directory')
+        backup_info.set_attribute('pgdata', data_directory)
 
-            # Set server version
-            backup_info.set_attribute('version', server.server_version)
+        # Set server version
+        backup_info.set_attribute('version', server.postgres.server_version)
 
-            # Set configuration files location
-            cf = server.get_pg_configuration_files()
-            if cf:
-                for key in sorted(cf):
-                    backup_info.set_attribute(key, cf[key])
+        # Set configuration files location
+        cf = server.postgres.get_configuration_files()
+        for key in cf:
+            backup_info.set_attribute(key, cf[key])
 
-            # Get tablespaces information
-            self.current_action = 'detecting tablespaces'
-            output.debug(self.current_action)
-            tablespaces = server.get_pg_tablespaces()
-            if tablespaces and len(tablespaces) > 0:
-                backup_info.set_attribute('tablespaces', tablespaces)
-                for item in tablespaces:
-                    msg = "\t%s, %s, %s" % (item.oid, item.name, item.location)
-                    _logger.info(msg)
+        # Get tablespaces information
+        self.current_action = 'detecting tablespaces'
+        output.debug(self.current_action)
+        tablespaces = server.postgres.get_tablespaces()
+        if tablespaces and len(tablespaces) > 0:
+            backup_info.set_attribute('tablespaces', tablespaces)
+            for item in tablespaces:
+                msg = "\t%s, %s, %s" % (item.oid, item.name, item.location)
+                _logger.info(msg)
 
 
 class ExclusiveBackupStrategy(BackupStrategy):
@@ -647,57 +700,6 @@ class ExclusiveBackupStrategy(BackupStrategy):
         assert (BackupOptions.CONCURRENT_BACKUP not in
                 self.executor.config.backup_options)
 
-    def _pg_start_backup(self, backup_label):
-        """
-        Calls pg_start_backup() on the PostgreSQL server
-
-        :param str backup_label: label for the backup returned by Postgres
-        :rtype: tuple
-        """
-        with self.executor.server.pg_connect() as conn:
-            try:
-                # Issue a rollback to release any unneeded lock
-                conn.rollback()
-                cur = conn.cursor()
-                if self.executor.server.server_version < 80400:
-                    cur.execute(
-                        "SELECT xlog_loc, "
-                        "(pg_xlogfile_name_offset(xlog_loc)).*, "
-                        "now() FROM pg_start_backup(%s) as xlog_loc",
-                        (backup_label,))
-                else:
-                    cur.execute(
-                        "SELECT xlog_loc, "
-                        "(pg_xlogfile_name_offset(xlog_loc)).*, "
-                        "now() FROM pg_start_backup(%s,%s) as xlog_loc",
-                        (backup_label,
-                         self.executor.config.immediate_checkpoint))
-                return cur.fetchone()
-            except psycopg2.Error, e:
-                msg = "pg_start_backup(): %s" % e
-                _logger.debug(msg)
-                raise Exception(msg)
-
-    def _pg_stop_backup(self):
-        """
-        Calls pg_stop_backup() on the PostgreSQL server
-
-        :returns: a tuple with the result of the pg_stop_backup() call or None
-        :rtype: tuple|None
-        """
-        with self.executor.server.pg_connect() as conn:
-            try:
-                # Issue a rollback to release any unneeded lock
-                conn.rollback()
-                cur = conn.cursor()
-                cur.execute(
-                    'SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).*, '
-                    'now() FROM pg_stop_backup() as xlog_loc')
-                return cur.fetchone()
-            except psycopg2.Error, e:
-                _logger.debug('Error issuing pg_stop_backup() command: %s', e)
-                return None
-
     def start_backup(self, backup_info):
         """
         Manage the start of an exclusive backup
@@ -711,29 +713,26 @@ class ExclusiveBackupStrategy(BackupStrategy):
         self.current_action = "connecting to database (%s)" % \
                               self.executor.config.conninfo
         output.debug(self.current_action)
-        server = self.executor.server
-        with server.pg_connect():
+        # Retrieve PostgreSQL server metadata
+        self._pg_get_metadata(backup_info)
 
-            # Retrieve PostgreSQL server metadata
-            self._pg_get_metadata(backup_info)
+        # Issue pg_start_backup on the PostgreSQL server
+        self.current_action = "issuing start backup command"
+        _logger.debug(self.current_action)
+        label = "Barman backup %s %s" % (
+            backup_info.server_name, backup_info.backup_id)
 
-            # Issue pg_start_backup on the PostgreSQL server
-            self.current_action = "issuing start backup command"
-            _logger.debug(self.current_action)
-            label = "Barman backup %s %s" % (
-                backup_info.server_name, backup_info.backup_id)
-
-            # Exclusive backup: issue a pg_start_Backup() command
-            start_row = self._pg_start_backup(label)
-            start_xlog, start_file_name, start_file_offset, start_time = \
-                start_row
-            backup_info.set_attribute('status', "STARTED")
-            backup_info.set_attribute('timeline',
-                                      int(start_file_name[0:8], 16))
-            backup_info.set_attribute('begin_xlog', start_xlog)
-            backup_info.set_attribute('begin_wal', start_file_name)
-            backup_info.set_attribute('begin_offset', start_file_offset)
-            backup_info.set_attribute('begin_time', start_time)
+        # Exclusive backup: issue a pg_start_Backup() command
+        start_row = self.executor.server.postgres.start_exclusive_backup(label)
+        start_xlog, start_file_name, start_file_offset, start_time = \
+            start_row
+        backup_info.set_attribute('status', "STARTED")
+        backup_info.set_attribute('timeline',
+                                  int(start_file_name[0:8], 16))
+        backup_info.set_attribute('begin_xlog', start_xlog)
+        backup_info.set_attribute('begin_wal', start_file_name)
+        backup_info.set_attribute('begin_offset', start_file_offset)
+        backup_info.set_attribute('begin_time', start_time)
 
     def stop_backup(self, backup_info):
         """
@@ -746,7 +745,8 @@ class ExclusiveBackupStrategy(BackupStrategy):
         :param barman.infofile.BackupInfo backup_info: backup information
         """
 
-        stop_row = self._pg_stop_backup()
+        self.current_action = "issuing stop backup command"
+        stop_row = self.executor.server.postgres.stop_exclusive_backup()
         if stop_row:
             stop_xlog, stop_file_name, stop_file_offset, stop_time = \
                 stop_row
@@ -767,14 +767,15 @@ class ExclusiveBackupStrategy(BackupStrategy):
              of the results of the various checks
         """
         # Make sure PostgreSQL is not in recovery (i.e. is a master)
-        is_in_recovery = self.executor.server.pg_is_in_recovery()
-        if not is_in_recovery:
-            check_strategy.result(
-                self.executor.config.name, 'not in recovery', True)
-        else:
-            check_strategy.result(
-                self.executor.config.name, 'not in recovery', False,
-                'cannot perform exclusive backup on a standby')
+        if self.executor.server.postgres:
+            is_in_recovery = self.executor.server.postgres.is_in_recovery
+            if not is_in_recovery:
+                check_strategy.result(
+                    self.executor.config.name, 'not in recovery', True)
+            else:
+                check_strategy.result(
+                    self.executor.config.name, 'not in recovery', False,
+                    'cannot perform exclusive backup on a standby')
 
 
 class ConcurrentBackupStrategy(BackupStrategy):
@@ -800,49 +801,6 @@ class ConcurrentBackupStrategy(BackupStrategy):
         assert (BackupOptions.CONCURRENT_BACKUP in
                 self.executor.config.backup_options)
 
-    def _pgespresso_start_backup(self, backup_label):
-        """
-        Execute a pgespresso_start_backup
-
-        :param str backup_label: label for the backup
-        :rtype: tuple
-        """
-        with self.executor.server.pg_connect() as conn:
-            try:
-                # Issue a rollback to release any unneeded lock
-                conn.rollback()
-                cur = conn.cursor()
-                cur.execute(
-                    'SELECT pgespresso_start_backup(%s,%s), now()',
-                    (backup_label, self.executor.config.immediate_checkpoint))
-                return cur.fetchone()
-            except psycopg2.Error, e:
-                msg = "pgespresso_start_backup(): %s" % e
-                _logger.debug(msg)
-                raise Exception(msg)
-
-    def _pgespresso_stop_backup(self, backup_label):
-        """
-        Execute a pgespresso_stop_backup
-
-        :param str backup_label: label of the backup
-        :returns: a string containing the result of the
-            pgespresso_stop_backup call or None
-        :rtype: tuple|None
-        """
-        with self.executor.server.pg_connect() as conn:
-            try:
-                # Issue a rollback to release any unneeded lock
-                conn.rollback()
-                cur = conn.cursor()
-                cur.execute("SELECT pgespresso_stop_backup(%s), now()",
-                            (backup_label,))
-                return cur.fetchone()
-            except psycopg2.Error, e:
-                _logger.debug(
-                    "Error issuing pgespresso_stop_backup() command: %s", e)
-                return None
-
     # noinspection PyMethodMayBeStatic
     def _write_backup_label(self, backup_info):
         """
@@ -868,35 +826,35 @@ class ConcurrentBackupStrategy(BackupStrategy):
         self.current_action = "connecting to database (%s)" % \
                               self.executor.config.conninfo
         output.debug(self.current_action)
-        with self.executor.server.pg_connect():
+        # with self.executor.server.pg_connect():
+        # Retrieve PostgreSQL server metadata
+        self._pg_get_metadata(backup_info)
 
-            # Retrieve PostgreSQL server metadata
-            self._pg_get_metadata(backup_info)
+        # Issue _pg_start_backup on the PostgreSQL server
+        self.current_action = "issuing start backup command"
+        _logger.debug(self.current_action)
+        label = "Barman backup %s %s" % (
+            backup_info.server_name, backup_info.backup_id)
 
-            # Issue _pg_start_backup on the PostgreSQL server
-            self.current_action = "issuing start backup command"
-            _logger.debug(self.current_action)
-            label = "Barman backup %s %s" % (
-                backup_info.server_name, backup_info.backup_id)
+        # Concurrent backup: issue a pgespresso_start_Backup() command
 
-            # Concurrent backup: issue a pgespresso_start_Backup() command
-
-            start_row = self._pgespresso_start_backup(label)
-            backup_data, start_time = start_row
-            wal_re = re.compile(
-                '^START WAL LOCATION: (.*) \(file (.*)\)',
-                re.MULTILINE)
-            wal_info = wal_re.search(backup_data)
-            backup_info.set_attribute('status', "STARTED")
-            backup_info.set_attribute('timeline',
-                                      int(wal_info.group(2)[0:8], 16))
-            backup_info.set_attribute('begin_xlog', wal_info.group(1))
-            backup_info.set_attribute('begin_wal', wal_info.group(2))
-            backup_info.set_attribute('begin_offset',
-                                      xlog.get_offset_from_location(
-                                          wal_info.group(1)))
-            backup_info.set_attribute('backup_label', backup_data)
-            backup_info.set_attribute('begin_time', start_time)
+        postgres = self.executor.server.postgres
+        start_row = postgres.pgespresso_start_backup(label)
+        backup_data, start_time = start_row
+        wal_re = re.compile(
+            '^START WAL LOCATION: (.*) \(file (.*)\)',
+            re.MULTILINE)
+        wal_info = wal_re.search(backup_data)
+        backup_info.set_attribute('status', "STARTED")
+        backup_info.set_attribute('timeline',
+                                  int(wal_info.group(2)[0:8], 16))
+        backup_info.set_attribute('begin_xlog', wal_info.group(1))
+        backup_info.set_attribute('begin_wal', wal_info.group(2))
+        backup_info.set_attribute('begin_offset',
+                                  xlog.get_offset_from_location(
+                                      wal_info.group(1)))
+        backup_info.set_attribute('backup_label', backup_data)
+        backup_info.set_attribute('begin_time', start_time)
 
     def stop_backup(self, backup_info):
         """
@@ -904,7 +862,8 @@ class ConcurrentBackupStrategy(BackupStrategy):
 
         :param barman.infofile.BackupInfo backup_info: backup information
         """
-        stop_row = self._pgespresso_stop_backup(backup_info.backup_label)
+        postgres = self.executor.server.postgres
+        stop_row = postgres.pgespresso_stop_backup(backup_info.backup_label)
         if stop_row:
             end_wal, stop_time = stop_row
             decoded_segment = xlog.decode_segment_name(end_wal)
@@ -930,7 +889,7 @@ class ConcurrentBackupStrategy(BackupStrategy):
         :param CheckStrategy check_strategy: the strategy for the management
              of the results of the various checks
         """
-        if self.executor.server.pgespresso_installed():
+        if self.executor.server.postgres.has_pgespresso:
             check_strategy.result(self.executor.config.name,
                                   'pgespresso extension', True)
         else:
