@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2015 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -18,25 +18,45 @@
 """
 This module implements the interface with the command line and the logger.
 """
+
 import logging
 import os
 import sys
-
-from argh import ArghParser, named, arg, expects_obj
 from argparse import SUPPRESS, ArgumentTypeError
+from contextlib import closing
 
+from argh import ArghParser, arg, expects_obj, named
+
+import barman.config
+import barman.diagnose
 from barman import output
 from barman.infofile import BackupInfo
 from barman.server import Server
-import barman.diagnose
-import barman.config
-from barman.utils import drop_privileges, configure_logging, parse_log_level
+from barman.utils import configure_logging, drop_privileges, parse_log_level
 from barman.xlog import BadXlogSegmentName
 
-
 _logger = logging.getLogger(__name__)
 
 
+def check_non_negative(value):
+    """
+    Check for a positive integer option
+
+    :param value: str containing the value to check
+    """
+    if value is None:
+        return None
+    try:
+        int_value = int(value)
+    except Exception:
+        raise ArgumentTypeError("'%s' is not a valid non negative integer" %
+                                value)
+    if int_value < 0:
+        raise ArgumentTypeError("'%s' is not a valid non negative integer" %
+                                value)
+    return int_value
+
+
 def check_positive(value):
     """
     Check for a positive integer option
@@ -48,9 +68,11 @@ def check_positive(value):
     try:
         int_value = int(value)
     except Exception:
-        raise ArgumentTypeError("'%s' is not a valid positive integer" % value)
-    if int_value < 0:
-        raise ArgumentTypeError("'%s' is not a valid positive integer" % value)
+        raise ArgumentTypeError("'%s' is not a valid positive integer" %
+                                value)
+    if int_value < 1:
+        raise ArgumentTypeError("'%s' is not a valid positive integer" %
+                                value)
     return int_value
 
 
@@ -155,10 +177,10 @@ def backup_completer(prefix, parsed_args, **kwargs):
           'If no argument is given "link" is assumed')
 @arg('--retry-times',
      help='Number of retries after an error if base backup copy fails.',
-     type=check_positive)
+     type=check_non_negative)
 @arg('--retry-sleep',
      help='Wait time after a failed base backup copy, before retrying.',
-     type=check_positive)
+     type=check_non_negative)
 @arg('--no-retry', help='Disable base backup copy retry logic.',
      dest='retry_times', action='store_const', const=0)
 @expects_obj
@@ -182,7 +204,8 @@ def backup(args):
             server.config.basebackup_retry_times = args.retry_times
         if hasattr(args, 'immediate_checkpoint'):
             server.config.immediate_checkpoint = args.immediate_checkpoint
-        server.backup()
+        with closing(server):
+            server.backup()
     output.close_and_exit()
 
 
@@ -206,7 +229,8 @@ def list_backup(args):
             continue
 
         output.init('list_backup', name, minimal=args.minimal)
-        server.list_backups()
+        with closing(server):
+            server.list_backups()
     output.close_and_exit()
 
 
@@ -227,7 +251,40 @@ def status(args):
             continue
 
         output.init('status', name)
-        server.status()
+        with closing(server):
+            server.status()
+    output.close_and_exit()
+
+
+@named('replication-status')
+@arg('server_name', nargs='+',
+     completer=server_completer_all,
+     help='specifies the server name for the command')
+@arg('--minimal', help='machine readable output', action='store_true')
+@arg('--target', choices=('all', 'hot-standby', 'wal-streamer'),
+     default='all',
+     help='''
+         Possible values are: 'hot-standby' (only hot standby servers),
+         'wal-streamer' (only WAL streaming clients, such as pg_receivexlog),
+         'all' (any of them). Defaults to %(default)s''')
+@expects_obj
+def replication_status(args):
+    """
+    Shows live information and status of any streaming client
+    """
+    servers = get_server_list(args, skip_inactive=True)
+    for name in sorted(servers):
+        server = servers[name]
+
+        # Skip the server (apply general rule)
+        if not manage_server_command(server, name):
+            continue
+
+        with closing(server):
+            output.init('replication_status',
+                        name,
+                        minimal=args.minimal)
+            server.replication_status(args.target)
     output.close_and_exit()
 
 
@@ -247,14 +304,15 @@ def rebuild_xlogdb(args):
         if not manage_server_command(server, name):
             continue
 
-        server.rebuild_xlogdb()
+        with closing(server):
+            server.rebuild_xlogdb()
     output.close_and_exit()
 
 
 @arg('server_name',
      completer=server_completer,
      help='specifies the server name for the command')
-@arg('--target-tli', help='target timeline', type=int)
+@arg('--target-tli', help='target timeline', type=check_positive)
 @arg('--target-time',
      help='target time. You can use any valid unambiguous representation. '
           'e.g: "YYYY-MM-DD HH:MM:SS.mmm"')
@@ -270,9 +328,10 @@ def rebuild_xlogdb(args):
 @arg('--remote-ssh-command',
      metavar='SSH_COMMAND',
      help='This options activates remote recovery, by specifying the secure '
-          'shell command to be launched on a remote host. It is the equivalent '
-          'of the "ssh_command" server option in the configuration file for '
-          'remote recovery. Example: "ssh postgres@db2"')
+          'shell command to be launched on a remote host. It is '
+          'the equivalent of the "ssh_command" server option in '
+          'the configuration file for remote recovery. '
+          'Example: "ssh postgres@db2"')
 @arg('backup_id',
      completer=backup_completer,
      help='specifies the backup ID to recover')
@@ -280,10 +339,10 @@ def rebuild_xlogdb(args):
      help='the directory where the new server is created')
 @arg('--retry-times',
      help='Number of retries after an error if base backup copy fails.',
-     type=check_positive)
+     type=check_non_negative)
 @arg('--retry-sleep',
      help='Wait time after a failed base backup copy, before retrying.',
-     type=check_positive)
+     type=check_non_negative)
 @arg('--no-retry', help='Disable base backup copy retry logic.',
      dest='retry_times', action='store_const', const=0)
 @expects_obj
@@ -316,8 +375,10 @@ def recover(args):
                 output.close_and_exit()
 
     # validate the rules against the tablespace list
-    valid_tablespaces = [tablespace_data.name for tablespace_data in
-                         backup_id.tablespaces] if backup_id.tablespaces else []
+    valid_tablespaces = []
+    if backup_id.tablespaces:
+        valid_tablespaces = [tablespace_data.name for tablespace_data in
+                             backup_id.tablespaces]
     for item in tablespaces:
         if item not in valid_tablespaces:
             output.error("Invalid tablespace name '%s'\n"
@@ -338,15 +399,16 @@ def recover(args):
         server.config.basebackup_retry_sleep = args.retry_sleep
     if args.retry_times is not None:
         server.config.basebackup_retry_times = args.retry_times
-    server.recover(backup_id,
-                   args.destination_directory,
-                   tablespaces=tablespaces,
-                   target_tli=args.target_tli,
-                   target_time=args.target_time,
-                   target_xid=args.target_xid,
-                   target_name=args.target_name,
-                   exclusive=args.exclusive,
-                   remote_command=args.remote_ssh_command)
+    with closing(server):
+        server.recover(backup_id,
+                       args.destination_directory,
+                       tablespaces=tablespaces,
+                       target_tli=args.target_tli,
+                       target_time=args.target_time,
+                       target_xid=args.target_xid,
+                       target_name=args.target_name,
+                       exclusive=args.exclusive,
+                       remote_command=args.remote_ssh_command)
 
     output.close_and_exit()
 
@@ -370,7 +432,33 @@ def show_server(args):
             continue
 
         output.init('show_server', name)
-        server.show()
+        with closing(server):
+            server.show()
+    output.close_and_exit()
+
+
+@named('switch-xlog')
+@arg('server_name', nargs='+',
+     completer=server_completer_all,
+     help="specifies the server name target of the switch-xlog command")
+@arg('--force',
+     help='forces the switch of a xlog by executing a checkpoint before',
+     dest='force',
+     action='store_true',
+     default=False)
+@expects_obj
+def switch_xlog(args):
+    """
+    Execute the switch-xlog command on the target server
+    """
+    servers = get_server_list(args, skip_inactive=True)
+    for name in sorted(servers):
+        server = servers[name]
+        # Skip the server (apply general rule)
+        if not manage_server_command(server, name):
+            continue
+        with closing(server):
+            server.switch_xlog(args.force)
     output.close_and_exit()
 
 
@@ -406,7 +494,8 @@ def check(args):
         elif server.config.disabled:
             name += " (WARNING: disabled)"
         output.init('check', name, server.config.active)
-        server.check()
+        with closing(server):
+            server.check()
     output.close_and_exit()
 
 
@@ -438,7 +527,8 @@ def show_backup(args):
 
     # Retrieves the backup
     backup_info = parse_backup_id(server, args)
-    server.show_backup(backup_info)
+    with closing(server):
+        server.show_backup(backup_info)
     output.close_and_exit()
 
 
@@ -465,9 +555,9 @@ def list_files(args):
     server = get_server(args)
 
     # Retrieves the backup
-    backup_id = parse_backup_id(server, args)
+    backup_info = parse_backup_id(server, args)
     try:
-        for line in backup_id.get_list_of_files(args.target):
+        for line in backup_info.get_list_of_files(args.target):
             output.info(line, log=False)
     except BadXlogSegmentName as e:
         output.error(
@@ -493,7 +583,8 @@ def delete(args):
 
     # Retrieves the backup
     backup_id = parse_backup_id(server, args)
-    server.delete_backup(backup_id)
+    with closing(server):
+        server.delete_backup(backup_id)
     output.close_and_exit()
 
 
@@ -504,14 +595,23 @@ def delete(args):
 @arg('wal_name',
      help='the WAL file to get')
 @arg('--output-directory', '-o',
-     help='put the retrieved WAL file in this directory with the original name',
+     help='put the retrieved WAL file in this directory '
+          'with the original name',
      default=SUPPRESS)
-@arg('--gzip', '-x',
+@arg('--gzip', '-z', '-x',
      help='compress the output with gzip',
      action='store_const', const='gzip', dest='compression', default=SUPPRESS)
 @arg('--bzip2', '-j',
      help='compress the output with bzip2',
      action='store_const', const='bzip2', dest='compression', default=SUPPRESS)
+@arg('--peek', '-p',
+     help="peek from the WAL archive up to 'SIZE' WAL files, starting "
+          "from the requested one. 'SIZE' must be an integer >= 1. "
+          "When invoked with this option, get-wal returns a list of "
+          "zero to 'SIZE' WAL segment names, one per row.",
+     metavar='SIZE',
+     type=check_positive,
+     default=SUPPRESS)
 @expects_obj
 def get_wal(args):
     """
@@ -520,14 +620,19 @@ def get_wal(args):
     the --output-directory option is specified.
     """
     server = get_server(args)
+
     # Retrieve optional arguments. If an argument is not specified,
     # the namespace doesn't contain it due to SUPPRESS default.
     # In that case we pick 'None' using getattr third argument.
     compression = getattr(args, 'compression', None)
     output_directory = getattr(args, 'output_directory', None)
-    server.get_wal(args.wal_name,
-                   compression=compression,
-                   output_directory=output_directory)
+    peek = getattr(args, 'peek', None)
+
+    with closing(server):
+        server.get_wal(args.wal_name,
+                       compression=compression,
+                       output_directory=output_directory,
+                       peek=peek)
     output.close_and_exit()
 
 
@@ -544,8 +649,37 @@ def archive_wal(args):
 
     """
     server = get_server(args)
-    output.debug("Starting archive-wal for server %s", server.config.name)
-    server.archive_wal()
+    with closing(server):
+        server.archive_wal()
+    output.close_and_exit()
+
+
+@named('receive-wal')
+@arg('--stop', help='stop the receive-wal subprocess for the server',
+     action='store_true')
+@arg('--reset', help='reset the status of receive-wal removing '
+                     'any status files',
+     action='store_true')
+@arg('server_name',
+     completer=server_completer,
+     help='specifies the server name for the command')
+@expects_obj
+def receive_wal(args):
+    """
+    Start a receive-wal process.
+    The process uses the streaming protocol to receive WAL files
+    from the PostgreSQL server.
+    """
+    server = get_server(args)
+    if args.stop and args.reset:
+        output.error("--stop and --reset options are not compatible")
+    # If the caller requested to shutdown the receive-wal process deliver the
+    # termination signal, otherwise attempt to start it
+    elif args.stop:
+        server.kill('receive-wal')
+    else:
+        with closing(server):
+            server.receive_wal(reset=args.reset)
     output.close_and_exit()
 
 
@@ -559,11 +693,11 @@ def pretty_args(args):
     values = dict(vars(args))
     # Retrieve the command name with recent argh versions
     if '_functions_stack' in values:
-        values['command'] = values['_functions_stack'][0].func_name
+        values['command'] = values['_functions_stack'][0].__name__
         del values['_functions_stack']
     # Older argh versions only have the matching function in the namespace
     elif 'function' in values:
-        values['command'] = values['function'].func_name
+        values['command'] = values['function'].__name__
         del values['function']
     return "%r" % values
 
@@ -626,7 +760,8 @@ def get_server(args, skip_inactive=True, skip_disabled=False,
 
     WARNING: this function modifies the 'args' parameter
 
-    :param args: an argparse namespace containing a single server_name parameter
+    :param args: an argparse namespace containing a single
+        server_name parameter
         WARNING: the function modifies the content of this parameter
     :param bool skip_inactive: skip inactive servers when 'all' is required
     :param bool skip_disabled: skip disabled servers when 'all' is required
@@ -734,17 +869,17 @@ def get_server_list(args=None, skip_inactive=False, skip_disabled=False,
             # Unknown server
             server_dict[server] = None
         else:
+            server_object = Server(conf)
             # Skip inactive servers, if requested
-            if skip_inactive and not conf.active:
+            if skip_inactive and not server_object.config.active:
                 output.info("Skipping inactive server '%s'"
                             % conf.name)
                 continue
             # Skip disabled servers, if requested
-            if skip_disabled and conf.disabled:
+            if skip_disabled and server_object.config.disabled:
                 output.info("Skipping temporarily disabled server '%s'"
                             % conf.name)
                 continue
-            server_object = Server(conf)
             server_dict[server] = server_object
 
     return server_dict
@@ -810,9 +945,9 @@ def parse_backup_id(server, args):
     :rtype: BackupInfo
     """
     if args.backup_id in ('latest', 'last'):
-        backup_id = server.get_last_backup()
+        backup_id = server.get_last_backup_id()
     elif args.backup_id in ('oldest', 'first'):
-        backup_id = server.get_first_backup()
+        backup_id = server.get_first_backup_id()
     else:
         backup_id = args.backup_id
     backup_info = server.get_backup(backup_id)
@@ -844,20 +979,23 @@ def main():
     p.add_commands(
         [
             archive_wal,
-            cron,
-            list_server,
-            show_server,
-            status,
-            check,
-            diagnose,
             backup,
-            list_backup,
-            show_backup,
-            list_files,
-            get_wal,
-            recover,
+            check,
+            cron,
             delete,
+            diagnose,
+            get_wal,
+            list_backup,
+            list_files,
+            list_server,
             rebuild_xlogdb,
+            receive_wal,
+            recover,
+            show_backup,
+            show_server,
+            replication_status,
+            status,
+            switch_xlog,
         ]
     )
     # noinspection PyBroadException
@@ -866,7 +1004,7 @@ def main():
     except KeyboardInterrupt:
         msg = "Process interrupted by user (KeyboardInterrupt)"
         output.exception(msg)
-    except Exception, e:
+    except Exception as e:
         msg = "%s\nSee log file for more details." % e
         output.exception(msg)
 
