@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2015 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -19,6 +19,7 @@
 This module is responsible for all the things related to
 Barman configuration, such as parsing configuration file.
 """
+
 import collections
 import datetime
 import inspect
@@ -26,11 +27,16 @@ import logging.handlers
 import os
 import re
 import sys
-from ConfigParser import ConfigParser, NoOptionError
 from glob import iglob
 
 from barman import output
 
+try:
+    from ConfigParser import ConfigParser, NoOptionError
+except ImportError:
+    from configparser import ConfigParser, NoOptionError
+
+
 # create a namedtuple object called PathConflict with 'label' and 'server'
 PathConflict = collections.namedtuple('PathConflict', 'label server')
 
@@ -43,8 +49,8 @@ DEFAULT_LOG_LEVEL = logging.INFO
 DEFAULT_LOG_FORMAT = "%(asctime)s [%(process)s] %(name)s " \
                      "%(levelname)s: %(message)s"
 
-_TRUE_RE = re.compile(r"""^(true|t|yes|1)$""", re.IGNORECASE)
-_FALSE_RE = re.compile(r"""^(false|f|no|0)$""", re.IGNORECASE)
+_TRUE_RE = re.compile(r"""^(true|t|yes|1|on)$""", re.IGNORECASE)
+_FALSE_RE = re.compile(r"""^(false|f|no|0|off)$""", re.IGNORECASE)
 _TIME_INTERVAL_RE = re.compile(r"""
       ^\s*
       (\d+)\s+(day|month|week)s?  # N (day|month|week) with optional 's'
@@ -54,7 +60,7 @@ _TIME_INTERVAL_RE = re.compile(r"""
 REUSE_BACKUP_VALUES = ('copy', 'link', 'off')
 
 # Possible copy methods for backups (must be all lowercase)
-COPY_METHOD_VALUES = ['rsync']
+BACKUP_METHOD_VALUES = ['rsync', 'postgres']
 
 
 class CsvOption(set):
@@ -234,22 +240,22 @@ def parse_reuse_backup(value):
             "', '".join(REUSE_BACKUP_VALUES[:-1]), REUSE_BACKUP_VALUES[-1]))
 
 
-def parse_copy_method(value):
+def parse_backup_method(value):
     """
-    Parse a string to a valid copy_method value.
+    Parse a string to a valid backup_method value.
 
-    Valid values are contained in COPY_METHOD_VALUES list
+    Valid values are contained in BACKUP_METHOD_VALUES list
 
-    :param str value: copy_method value
+    :param str value: backup_method value
     :raises ValueError: if the value is invalid
     """
     if value is None:
         return None
-    if value.lower() in COPY_METHOD_VALUES:
+    if value.lower() in BACKUP_METHOD_VALUES:
         return value.lower()
     raise ValueError(
         "Invalid value (must be one in: '%s')" % (
-            "', '".join(COPY_METHOD_VALUES)))
+            "', '".join(BACKUP_METHOD_VALUES)))
 
 
 class ServerConfig(object):
@@ -258,58 +264,102 @@ class ServerConfig(object):
     """
 
     KEYS = [
-        'active', 'description', 'ssh_command', 'conninfo',
-        'backup_directory', 'basebackups_directory', 'disabled',
-        'wals_directory', 'incoming_wals_directory',
-        'compression', 'custom_compression_filter',
-        'custom_decompression_filter', 'retention_policy_mode',
-        'retention_policy',
-        'wal_retention_policy', 'pre_backup_script', 'post_backup_script',
-        'pre_backup_retry_script', 'post_backup_retry_script',
-        'pre_archive_script', 'post_archive_script',
-        'pre_archive_retry_script', 'post_archive_retry_script',
-        'minimum_redundancy', 'bandwidth_limit', 'tablespace_bandwidth_limit',
-        'reuse_backup',
-        'backup_options', 'immediate_checkpoint', 'network_compression',
-        'recovery_options',
-        'basebackup_retry_times', 'basebackup_retry_sleep',
+        'active',
+        'archiver',
+        'backup_directory',
+        'backup_method',
+        'backup_options',
+        'bandwidth_limit',
+        'basebackup_retry_sleep',
+        'basebackup_retry_times',
+        'basebackups_directory',
+        'compression',
+        'conninfo',
+        'custom_compression_filter',
+        'custom_decompression_filter',
+        'description',
+        'disabled',
+        'errors_directory',
+        'immediate_checkpoint',
+        'incoming_wals_directory',
         'last_backup_maximum_age',
-        'copy_method',
+        'minimum_redundancy',
+        'network_compression',
+        'path_prefix',
+        'post_archive_retry_script',
+        'post_archive_script',
+        'post_backup_retry_script',
+        'post_backup_script',
+        'pre_archive_retry_script',
+        'pre_archive_script',
+        'pre_backup_retry_script',
+        'pre_backup_script',
+        'recovery_options',
+        'retention_policy',
+        'retention_policy_mode',
+        'reuse_backup',
+        'ssh_command',
+        'streaming_archiver',
+        'streaming_archiver_name',
+        'streaming_conninfo',
+        'streaming_wals_directory',
+        'tablespace_bandwidth_limit',
+        'wal_retention_policy',
+        'wals_directory'
     ]
 
     BARMAN_KEYS = [
-        'compression', 'custom_compression_filter',
-        'custom_decompression_filter', 'retention_policy_mode',
-        'retention_policy',
-        'wal_retention_policy', 'pre_backup_script', 'post_backup_script',
-        'pre_archive_script', 'post_archive_script',
+        'archiver',
+        'backup_method',
+        'backup_options',
+        'bandwidth_limit',
+        'basebackup_retry_sleep',
+        'basebackup_retry_times',
+        'compression',
         'configuration_files_directory',
-        'minimum_redundancy', 'bandwidth_limit', 'tablespace_bandwidth_limit',
-        'reuse_backup',
-        'backup_options', 'immediate_checkpoint', 'network_compression',
-        'recovery_options',
-        'basebackup_retry_times', 'basebackup_retry_sleep',
+        'custom_compression_filter',
+        'custom_decompression_filter',
+        'immediate_checkpoint',
         'last_backup_maximum_age',
-        'copy_method',
+        'minimum_redundancy',
+        'network_compression',
+        'path_prefix',
+        'post_archive_script',
+        'post_backup_script',
+        'pre_archive_script',
+        'pre_backup_script',
+        'recovery_options',
+        'retention_policy',
+        'retention_policy_mode',
+        'reuse_backup',
+        'streaming_archiver',
+        'tablespace_bandwidth_limit',
+        'wal_retention_policy'
     ]
 
     DEFAULTS = {
         'active': 'true',
-        'disabled': 'false',
-        'backup_directory': r'%(barman_home)s/%(name)s',
-        'basebackups_directory': r'%(backup_directory)s/base',
-        'wals_directory': r'%(backup_directory)s/wals',
-        'incoming_wals_directory': r'%(backup_directory)s/incoming',
-        'retention_policy_mode': 'auto',
-        'wal_retention_policy': 'main',
-        'minimum_redundancy': '0',
+        'archiver': 'on',
+        'backup_directory': '%(barman_home)s/%(name)s',
+        'backup_method': 'rsync',
         'backup_options': "%s" % BackupOptions.EXCLUSIVE_BACKUP,
-        'recovery_options': '',
-        'immediate_checkpoint': 'false',
-        'network_compression': 'false',
-        'basebackup_retry_times': '0',
         'basebackup_retry_sleep': '30',
-        'copy_method': 'rsync',
+        'basebackup_retry_times': '0',
+        'basebackups_directory': '%(backup_directory)s/base',
+        'disabled': 'false',
+        'errors_directory': '%(backup_directory)s/errors',
+        'immediate_checkpoint': 'false',
+        'incoming_wals_directory': '%(backup_directory)s/incoming',
+        'minimum_redundancy': '0',
+        'network_compression': 'false',
+        'recovery_options': '',
+        'retention_policy_mode': 'auto',
+        'streaming_archiver': 'off',
+        'streaming_archiver_name': 'barman_receive_wal',
+        'streaming_conninfo': '%(conninfo)s',
+        'streaming_wals_directory': '%(backup_directory)s/streaming',
+        'wal_retention_policy': 'main',
+        'wals_directory': '%(backup_directory)s/wals'
     }
 
     FIXED = [
@@ -318,16 +368,18 @@ class ServerConfig(object):
 
     PARSERS = {
         'active': parse_boolean,
+        'archiver': parse_boolean,
+        'backup_method': parse_backup_method,
+        'backup_options': BackupOptions,
+        'basebackup_retry_sleep': int,
+        'basebackup_retry_times': int,
         'disabled': parse_boolean,
         'immediate_checkpoint': parse_boolean,
-        'network_compression': parse_boolean,
-        'backup_options': BackupOptions,
-        'reuse_backup': parse_reuse_backup,
-        'basebackup_retry_times': int,
-        'basebackup_retry_sleep': int,
-        'recovery_options': RecoveryOptions,
         'last_backup_maximum_age': parse_time_interval,
-        'copy_method': parse_copy_method,
+        'network_compression': parse_boolean,
+        'recovery_options': RecoveryOptions,
+        'reuse_backup': parse_reuse_backup,
+        'streaming_archiver': parse_boolean,
     }
 
     def invoke_parser(self, key, source, value, new_value):
@@ -364,7 +416,7 @@ class ServerConfig(object):
                     value = parser(new_value, key, source)
                 else:
                     value = parser(new_value)
-            except Exception, e:
+            except Exception as e:
                 output.warning("Invalid configuration value '%s' for key %s"
                                " in %s: %s",
                                value, key, source, e)
@@ -407,7 +459,7 @@ class ServerConfig(object):
                 value = self.invoke_parser(key, source, value, new_value)
             # An empty string is a None value (bypassing inheritance
             # from global configuration)
-            if value is not None and value == '':
+            if value is not None and value == '' or value == 'None':
                 value = None
             setattr(self, key, value)
 
@@ -490,7 +542,8 @@ class Config(object):
             'barman', 'barman_lock_directory') or self.barman_home
         self.user = self.get('barman', 'barman_user') or DEFAULT_USER
         self.log_file = self.get('barman', 'log_file')
-        self.log_format = self.get('barman', 'log_format') or DEFAULT_LOG_FORMAT
+        self.log_format = self.get(
+            'barman', 'log_format') or DEFAULT_LOG_FORMAT
         self.log_level = self.get('barman', 'log_level') or DEFAULT_LOG_LEVEL
         # save the raw barman section to be compared later in
         # _is_global_config_changed() method
@@ -588,14 +641,22 @@ class Config(object):
             # Paths map
             section_conf = self._servers[section]
             config_paths = {
-                'backup_directory': section_conf.backup_directory,
-                'basebackups_directory': section_conf.basebackups_directory,
-                'wals_directory': section_conf.wals_directory,
-                'incoming_wals_directory': section_conf.incoming_wals_directory,
+                'backup_directory':
+                    section_conf.backup_directory,
+                'basebackups_directory':
+                    section_conf.basebackups_directory,
+                'errors_directory':
+                    section_conf.errors_directory,
+                'incoming_wals_directory':
+                    section_conf.incoming_wals_directory,
+                'streaming_wals_directory':
+                    section_conf.streaming_wals_directory,
+                'wals_directory':
+                    section_conf.wals_directory,
             }
 
             # Check for path errors
-            for label, path in sorted(config_paths.iteritems()):
+            for label, path in sorted(config_paths.items()):
                 # If the path does not conflict with the others, add it to the
                 # paths map
                 real_path = os.path.realpath(path)
@@ -711,13 +772,13 @@ class Config(object):
 # easy raw config diagnostic with python -m
 # noinspection PyProtectedMember
 def _main():
-    print "Active configuration settings:"
+    print("Active configuration settings:")
     r = Config()
     r.load_configuration_files_directory()
     for section in r._config.sections():
-        print "Section: %s" % section
+        print("Section: %s" % section)
         for option in r._config.options(section):
-            print "\t%s = %s " % (option, r.get(section, option))
+            print("\t%s = %s " % (option, r.get(section, option)))
 
 
 if __name__ == "__main__":
