@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 2ndQuadrant Italia Srl
+# Copyright (C) 2013-2016 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -14,13 +14,15 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>.
-import ast
 
-import os
-import dateutil.parser
-import dateutil.tz
+import ast
 import collections
 import logging
+import os
+
+import dateutil.parser
+import dateutil.tz
+
 from barman import xlog
 from barman.compression import identify_compression
 
@@ -302,7 +304,7 @@ class WalFileInfo(FieldListFile):
     Metadata of a WAL file.
     """
 
-    __slots__ = ()
+    __slots__ = ('orig_filename',)
 
     name = Field('name', doc='base name of WAL file')
     size = Field('size', load=int, doc='WAL file size after compression')
@@ -332,6 +334,7 @@ class WalFileInfo(FieldListFile):
                 or default_compression
         obj = cls(**kwargs)
         obj.filename = "%s.meta" % filename
+        obj.orig_filename = filename
         return obj
 
     def to_xlogdb_line(self):
@@ -411,6 +414,7 @@ class BackupInfo(FieldListFile):
     DONE = 'DONE'
     STATUS_ALL = (EMPTY, STARTED, DONE, FAILED)
     STATUS_NOT_EMPTY = (STARTED, DONE, FAILED)
+    STATUS_ARCHIVING = (STARTED, DONE)
 
     #: Status according to retention policies
     OBSOLETE = 'OBSOLETE'
@@ -452,7 +456,6 @@ class BackupInfo(FieldListFile):
                  'backup_id', 'backup_version')
 
     def __init__(self, server, info_file=None, backup_id=None, **kwargs):
-        # Initialises the attributes for the object based on the predefined keys
         """
         Stores meta information about a single backup
 
@@ -462,7 +465,8 @@ class BackupInfo(FieldListFile):
         :raise BackupInfoBadInitialisation: if the info_file content is invalid
             or neither backup_info or
         """
-
+        # Initialises the attributes for the object
+        # based on the predefined keys
         super(BackupInfo, self).__init__(**kwargs)
 
         self.server = server
@@ -500,15 +504,15 @@ class BackupInfo(FieldListFile):
                     os.path.join(self.get_basebackup_directory(), 'pgdata')):
                 self.backup_version = 1
         except Exception as e:
-            _logger.warning("Error detecting backup_version, use default: 2.\n "
-                            "Failure reason: %s", e)
+            _logger.warning("Error detecting backup_version, "
+                            "use default: 2. Failure reason: %s", e)
 
     def get_required_wal_segments(self):
         """
         Get the list of required WAL segments for the current backup
         """
-        return xlog.enumerate_segments(self.begin_wal, self.end_wal,
-                                       self.version)
+        return xlog.generate_segment_names(self.begin_wal, self.end_wal,
+                                           self.version)
 
     def get_list_of_files(self, target):
         """
@@ -602,7 +606,8 @@ class BackupInfo(FieldListFile):
             dir_name = os.path.dirname(filename)
             if not os.path.exists(dir_name):
                 os.makedirs(dir_name)
-        super(BackupInfo, self).save(filename=filename, file_object=file_object)
+        super(BackupInfo, self).save(filename=filename,
+                                     file_object=file_object)
 
     def to_dict(self):
         """
