@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 2ndQuadrant Italia Srl
+# Copyright (C) 2013-2016 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -19,13 +19,15 @@
 This module control how the output of Barman will be rendered
 """
 
+from __future__ import print_function
+
 import inspect
 import logging
 import sys
+
 from barman.infofile import BackupInfo
 from barman.utils import pretty_size
 
-
 __all__ = [
     'error_occurred', 'debug', 'info', 'warning', 'error', 'exception',
     'result', 'close_and_exit', 'close', 'set_output_writer',
@@ -112,7 +114,8 @@ def _dispatch(obj, prefix, name, *args, **kwargs):
     :param obj: the target object
     :param str prefix: prefix of the method to be called
     :param str name: name of the method to be called
-    :param tuple args: all remaining positional arguments will be sent to target
+    :param tuple args: all remaining positional arguments will be sent
+        to target
     :param dict kwargs: all remaining keyword arguments will be sent to target
     :return: the result of the invoked method
     :raise ValueError: if the target method is not present
@@ -174,7 +177,8 @@ def warning(message, *args, **kwargs):
 def error(message, *args, **kwargs):
     """
     Output a message with severity 'ERROR'.
-    Also records that an error has occurred unless the ignore parameter is True.
+    Also records that an error has occurred unless the ignore parameter
+    is True.
 
     :key bool ignore: avoid setting an error exit status (default False)
     :key bool log: whether to log the message
@@ -324,13 +328,13 @@ class ConsoleOutputWriter(object):
         """
         Print a message on standard output
         """
-        print >> sys.stdout, _format_message(message, args)
+        print(_format_message(message, args), file=sys.stdout)
 
     def _err(self, message, args):
         """
         Print a message on standard error
         """
-        print >> sys.stderr, _format_message(message, args)
+        print(_format_message(message, args), file=sys.stderr)
 
     def is_quiet(self):
         """
@@ -534,7 +538,8 @@ class ConsoleOutputWriter(object):
                              pretty_size(backup_size),
                              pretty_size(wal_size)))
             if backup_info.tablespaces:
-                tablespaces = [("%s:%s" % (tablespace.name, tablespace.location))
+                tablespaces = [("%s:%s" % (tablespace.name,
+                                           tablespace.location))
                                for tablespace in backup_info.tablespaces]
                 out_list.append(' (tablespaces: %s)' %
                                 ', '.join(tablespaces))
@@ -548,9 +553,11 @@ class ConsoleOutputWriter(object):
         """
         Output all available information about a backup in show-backup command
 
-        The argument has to be the result of a Server.get_backup_ext_info() call
+        The argument has to be the result
+        of a Server.get_backup_ext_info() call
 
-        :param dict backup_ext_info: a dictionary containing the info to display
+        :param dict backup_ext_info: a dictionary containing
+            the info to display
         """
         data = dict(backup_ext_info)
         self.info("Backup %s:", data['backup_id'])
@@ -609,9 +616,10 @@ class ConsoleOutputWriter(object):
                           data['wals_per_second'] * 3600)
             # Output WAL compression ratio for archived WAL files
             if data['wal_until_next_compression_ratio'] > 0:
-                self.info("    Compression ratio    : %s",
-                          '{percent:.2%}'.format(
-                              percent=data['wal_until_next_compression_ratio']))
+                self.info(
+                    "    Compression ratio    : %s",
+                    '{percent:.2%}'.format(
+                        percent=data['wal_until_next_compression_ratio']))
             self.info("    Last available       : %s", data['wal_last'])
             self.info("")
             self.info("  Catalog information:")
@@ -654,6 +662,166 @@ class ConsoleOutputWriter(object):
             description=description, message=message))
         self.info("\t%s: %s", description, message)
 
+    def init_replication_status(self, server_name, minimal=False):
+        """
+        Init the 'standby-status' command
+
+        :param str server_name: the server we are start listing
+        :param str minimal: minimal output
+        """
+        self.minimal = minimal
+
+    def result_replication_status(self, server_name, target, xlog_location,
+                                  standby_info):
+        """
+        Record a result line of a server status command
+
+        and output it as INFO
+
+        :param str server_name: the replication server
+        :param str target: all|hot-standby|wal-streamer
+        :param str xlog_location: server's xlog location
+        :param StatReplication standby_info: status info of a standby
+        """
+
+        if target == 'hot-standby':
+            title = 'hot standby servers'
+        elif target == 'wal-streamer':
+            title = 'WAL streamers'
+        else:
+            title = 'streaming clients'
+
+        if self.minimal:
+            # Minimal output
+            if xlog_location:
+                # xlog location from the master
+                self.info("%s for master '%s' (xlog @ %s):",
+                          title.capitalize(), server_name, xlog_location)
+            else:
+                # We are connected to a standby
+                self.info("%s for slave '%s':",
+                          title.capitalize(), server_name)
+        else:
+            # Full output
+            self.info("Status of %s for server '%s':",
+                      title, server_name)
+            # xlog location from the master
+            if xlog_location:
+                self.info("  Current xlog location on master: %s",
+                          xlog_location)
+
+        if standby_info is not None and not len(standby_info):
+            self.info("  No %s attached", title)
+            return
+
+        # Minimal output
+        if self.minimal:
+            n = 1
+            for standby in standby_info:
+                if not standby.replay_location:
+                    # WAL streamer
+                    self.info("  %s. W) %s@%s S:%s W:%s P:%s AN:%s",
+                              n,
+                              standby.usename,
+                              standby.client_addr or 'socket',
+                              standby.sent_location,
+                              standby.write_location,
+                              standby.sync_priority,
+                              standby.application_name)
+                else:
+                    # Standby
+                    self.info("  %s. %s) %s@%s S:%s F:%s R:%s P:%s AN:%s",
+                              n,
+                              standby.sync_state[0].upper(),
+                              standby.usename,
+                              standby.client_addr or 'socket',
+                              standby.sent_location,
+                              standby.flush_location,
+                              standby.replay_location,
+                              standby.sync_priority,
+                              standby.application_name)
+                n += 1
+        else:
+            n = 1
+            self.info("  Number of %s: %s",
+                      title, len(standby_info))
+            for standby in standby_info:
+                self.info("")
+
+                # Determine the sync stage of the client
+                sync_stage = None
+                if not standby.replay_location:
+                    client_type = 'WAL streamer'
+                    max_level = 3
+                else:
+                    client_type = 'standby'
+                    max_level = 5
+                    # Only standby can replay WAL info
+                    if standby.replay_diff == 0:
+                        sync_stage = '5/5 Hot standby (max)'
+                    elif standby.flush_diff == 0:
+                        sync_stage = '4/5 2-safe'  # remote flush
+
+                # If not yet done, set the sync stage
+                if not sync_stage:
+                    if standby.write_diff == 0:
+                        sync_stage = '3/%s Remote write' % max_level
+                    elif standby.sent_diff == 0:
+                        sync_stage = '2/%s WAL Sent (min)' % max_level
+                    else:
+                        sync_stage = '1/%s 1-safe' % max_level
+
+                # Synchronous standby
+                if standby.sync_priority > 0:
+                    self.info("  %s. #%s %s %s",
+                              n,
+                              standby.sync_priority,
+                              standby.sync_state.capitalize(),
+                              client_type)
+                # Asynchronous standby
+                else:
+                    self.info("  %s. %s %s",
+                              n,
+                              standby.sync_state.capitalize(),
+                              client_type)
+                self.info("     Application name: %s",
+                          standby.application_name)
+                self.info("     Sync stage      : %s",
+                          sync_stage)
+                if standby.client_addr:
+                    self.info("     Communication   : TCP/IP")
+                    self.info("     IP Address      : %s "
+                              "/ Port: %s / Host: %s",
+                              standby.client_addr,
+                              standby.client_port,
+                              standby.client_hostname or '-')
+                else:
+                    self.info("     Communication   : Unix domain socket")
+                self.info("     User name       : %s", standby.usename)
+                self.info("     Current state   : %s (%s)",
+                          standby.state,
+                          standby.sync_state)
+                self.info("     WAL sender PID  : %s", standby.pid)
+                self.info("     Started at      : %s", standby.backend_start)
+                if standby.backend_xmin:
+                    self.info("     Standby's xmin  : %s",
+                              standby.backend_xmin or '-')
+                self.info("     Sent location   : %s (diff: %s)",
+                          standby.sent_location,
+                          pretty_size(standby.sent_diff))
+                self.info("     Write location  : %s (diff: %s)",
+                          standby.write_location,
+                          pretty_size(standby.write_diff))
+                if standby.flush_location:
+                    self.info("     Flush location  : %s (diff: %s)",
+                              standby.flush_location,
+                              pretty_size(standby.flush_diff))
+                if standby.replay_location:
+                    self.info("     Replay location : %s (diff: %s)",
+                              standby.replay_location,
+                              pretty_size(standby.replay_diff))
+                n += 1
+
     def init_list_server(self, server_name, minimal=False):
         """
         Init the list-server command
@@ -734,7 +902,7 @@ class NagiosOutputWriter(ConsoleOutputWriter):
 
         # Global error (detected at configuration level)
         if len(issues) == 0 and error_occurred:
-            print "BARMAN CRITICAL - Global configuration errors"
+            print("BARMAN CRITICAL - Global configuration errors")
             error_exit_code = 2
             return
 
@@ -764,37 +932,39 @@ class NagiosOutputWriter(ConsoleOutputWriter):
                 # .....
                 for issue in self.result_check_list:
                     if issue['server_name'] == server and not issue['status']:
-                        fail_detail = "%s.%s: FAILED" % (server, issue['check'])
+                        fail_detail = "%s.%s: FAILED" % (server,
+                                                         issue['check'])
                         if issue['hint']:
                             fail_detail += " (%s)" % issue['hint']
                         details.append(fail_detail)
             # Append the summary of failures to the first line of the output
             # using * as delimiter
             if len(servers) == 1:
-                print "BARMAN CRITICAL - server %s has issues * %s" % \
-                    (servers[0], " * ".join(fail_summary))
+                print("BARMAN CRITICAL - server %s has issues * %s" %
+                      (servers[0], " * ".join(fail_summary)))
             else:
-                print "BARMAN CRITICAL - %d server out of %d have issues * " \
+                print("BARMAN CRITICAL - %d server out of %d have issues * "
                       "%s" % (len(issues), len(servers),
-                              " * ".join(fail_summary))
+                              " * ".join(fail_summary)))
 
             # add the detailed list to the output
             for issue in details:
-                print issue
+                print(issue)
             error_exit_code = 2
         else:
             # No issues, all good!
             # Display the output message for a single server check
             if len(servers) == 1:
-                print "BARMAN OK - Ready to serve the Espresso backup " \
-                    "for %s" % \
-                    (servers[0])
+                print("BARMAN OK - Ready to serve the Espresso backup "
+                      "for %s" %
+                      (servers[0]))
             else:
                 # Display the output message for several servers, using
                 # '*' as delimiter
-                print "BARMAN OK - Ready to serve the Espresso backup " \
-                    "for %d server(s) * %s" % \
-                    (len(servers), " * ".join([server for server in servers]))
+                print("BARMAN OK - Ready to serve the Espresso backup "
+                      "for %d server(s) * %s" % (
+                          len(servers),
+                          " * ".join([server for server in servers])))
 
 
 #: This dictionary acts as a registry of available OutputWriters
